
//********************************************************************//
//	Libreria per la gestione di tastiere a matrice
// spunto da Niccoli Damiano 2008
// modificata da pao per PIC18F4550 e per l'uso delle R di pull-up
// della PORTB (attivate nel main)
// RB1..RB3 collegate alle colonne (pin 1,2,3 della keypad)
// RB4..RB7 collegate alle righe
//
//  Contiene metodi di base per l'inizializzazione della porta 
//	e per la ridefinizione del layout dei pulsanti. Allo stato
//	attuale, si utilizza una sola porta: il nibble alto  in
//	uscita, il nibble basso  in ingresso per la lettura del
//	tasto premuto.
//
//	KEYB_INIT(); Da chiamare all'inizio del programma, configura la porta
//
//	KEYB_READ(MODE); Da chiamare ogni qual volta si intenda effettuare una
//				 lettura della tastiera. Il metodo ha due possibilit di
//				 funzionamento: bloccante (LOCK) e non bloccante (OPEN).
//				 Nel primo caso, il metodo non ritorna alcun valore fino 
//				 a quando non viene premuto un tasto; questo metodo comprende
//				 anche l'antirimbalzo a stato e il valore di ritorno viene 
//				 trasmesso al rilascio del pulsante.
//				 Nel secondo caso, la chiamata al metodo ritorna comunque 
//				 un valore al termine della scansione: 0, se non  stato premuto
//				 alcun tasto, altrimenti il carattere corrispondente.
//
//************************************************************************//

//************************************************************************//
// File da includere
//***********************************************************************//
#include	"p18cxxx.h"
#include	"keyb.h"

//************************************************************************//
// Definizione porta per controllo tastiera a matrice 
//************************************************************************//
#define DATA_PORT PORTB
#define LAT_PORT LATB

// 
#define COL_1 PORTBbits.RB0
#define COL_2 PORTBbits.RB1
#define COL_3 PORTBbits.RB2
#define COL_4 PORTBbits.RB3


#define NTASTI 16
#define NOTASTO 0xFF
#define OPEN 0
#define LOCK 1
#define ROW_NIBBLE	0xF0
#define COL_NIBBLE 	0x0F

//************************************************************************//

unsigned char DATA_MASK;
unsigned char nibbleL;
char const layout[NTASTI] = {'1','2','3','A',
                             '4','5','6','B',
                             '7','8','9','C',
                             '#','0','*','D'};

//************************************************************************//
// Metodo per l'inizializzazione della tastiera
//************************************************************************//

void KEYB_INIT (void)
{
	//TRIS_PORT = 0x0F;		gia' nel main
	LAT_PORT |= 0xF0;	// righe inizialmente alte
}
//************************************************************************//

//************************************************************************//
// Metodo per la scansione della tastiera
// se  stato premuto un tasto ritorna il carattere corrispondente,
// altrimenti ritorna la costante NOTASTO
//************************************************************************//

char KEYB_SCAN (void) {	
		unsigned char iii = 0;	//indice caratteri in layout[]
		unsigned char iriga = 0;
						
		while (iii < NTASTI){
			// mette iriga bassa, le altre alte
			switch (iriga) {
				case 0: LAT_PORT = 0b11101111; break;
				case 1: LAT_PORT = 0b11011111; break;
				case 2: LAT_PORT = 0b10111111; break;
				case 3: LAT_PORT = 0b01111111;
			}	
			delays_ms(5);
			// legge le colonne
			if (!COL_1) return layout[iii];
			iii++;	
			if (!COL_2)	return layout[iii];
			iii++;
			if (!COL_3) return layout[iii];
			iii++;
			if (!COL_4) return layout[iii];
			iii++;
			
			LAT_PORT = 0xFF;	// mette le righe tutte alte
			// incrementa la riga da 0 a 3
			iriga=(iriga+1) % 4;
		}
		return NOTASTO;		// indica nessun tasto premuto
}

//************************************************************************//
// Metodo per il ritorno del carattere e antirimbalzo
//************************************************************************//

unsigned char KEYB_READ (unsigned char MODE) {
	
	char kkk;

	if (MODE){	//modo LOCK==1 bloccante
		// scansiona finch non riceve un carattere
		do
			kkk =	KEYB_SCAN();
		while (kkk==NOTASTO);
		// attendi finch le colonne non tornano a riposo (tutte alte)
		while ((COL_1 && COL_2 && COL_3 && COL_4)==0);			//ANTIRIMBALZO A STATO
	
		// attendi altri 10 ms
		delays_ms(10);								//ANTIRIMBALZO A TEMPO
		
		return kkk;
	}
	else {		// modo OPEN==0, non bloccante
		kkk =	KEYB_SCAN();
		if (kkk!=NOTASTO) 	//  stato premuto un tasto
			// attendi finch le colonne non tornano a riposo (tutte alte)
			while ((COL_1 && COL_2 && COL_3 && COL_4)==0);
			delays_ms(5);
			while ((COL_1 && COL_2 && COL_3 && COL_4)==0);
		return kkk;
	}		

}

//************************************************************************//

