/* 18_Telecomando
mainRX.c  - PIC del ricevitore
versione implementataa come state machine
Paolo Reg 26/4/2011
spunti: 
	Ken Shriff's blog - A Multi-Protocol Infrared Remote Library for the Arduino
	http://dev.sarasini.it/?p=224  Telecomando IR con Pic

modulo ricevitore Vishay TSOP34838 (38 kHz)

out del ricevitore su RC0
led su RC1 lampeggia a frequenza overflow di TMR0
led su RC2 replica l'out del ricevitore
display su PORTD
*/

#pragma config FOSC=HS,LVP=OFF,WDT=OFF
#pragma config PWRT = ON,PBADEN=OFF
#pragma config CCP2MX = ON		//modulo CCP2 su RC1

// -- DEFINE --
#define LED1 LATCbits.LATC1		// 
#define LED2 LATCbits.LATC2		// replica RECEIV
#define RECEIV PORTCbits.RC0	// ingresso dal modulo receiver (attivo basso)
#define MAXNR 20	// n.max durate impulsi da registrare=dimensione array durate[]	

#define NOCOMMAND 255
#define DISP_TK 20000 	// ogni quanti 50us visualizzare i dati (1s)
#define MARK 0
#define SPACE 1 

// stati per la state machine
#define SM_IDLE 2
#define SM_MARK 3
#define SM_SPACE 4
#define SM_STOP 5

// -- VARIABILI GLOBALI --
volatile unsigned int iOVF=0,gapTK;	// iOVF contatore di overflow di TMR1
volatile unsigned char i,StatoReceiv,TastoRil,SMstato;
unsigned char idurate=0,idurC=0;
unsigned char durate[MAXNR]={0};

// -- INCLUDE --
#include <p18f4550.h>
#include <stdio.h>
#include <timers.h>
// usa Delay1KTCYx(n) ritardo (0,2 x n) ms, Delay10KTCYx(n) ritardo (2 x n) ms
#include <delays.h>	
#include <usart.h>

// libreria LCD
void delays_ms(unsigned long);	//richiamata in LCD_44780.c e keyb.c
#define LCD_DEFAULT
#include "LCD_44780.c"
//
// libreria del telecomando
// in telecSM.h ci sono le specifiche del telecomando e le costanti
//#define modoSM		//compila la libreia in modo State Machine	
#include "telecSM.h"

// --PROTOTIPI FUNZIONI
void Resume(void);
unsigned char Decode(void);
void setup(void);
// -- PROTOTIPI FUNZIONI locali alla libreria --
void ScriviSeq(unsigned char durate[],unsigned char ndurate);
unsigned char DecodSeq(unsigned char durate[],unsigned char Commands[]);
unsigned char Confronta(unsigned char cosa,unsigned char concosa,unsigned char Tol);


// -- ISR - routines interruzioni --
// prototipo funzione che gestisce l'interrupt ad alta priorit
void hInt_Event(void);

// l'interruzione fa saltare alla locazione 0x08, detta interrupt vector 
#pragma code h_int_vect=0x08
void h_int(void) {
	_asm GOTO hInt_Event _endasm
}
#pragma code

// dichiaro che la funzione di gestione interrupt si chiama hInt_Event
#pragma interrupt hInt_Event

// definizione della funzione di gestione int (ad alta priorit, modo compatibile)
void hInt_Event(void) {
	if (INTCONbits.TMR0IF == 1) {	//  avvenuto un overflow in TMR0
		INTCONbits.TMR0IF = 0;	// resetto l'IF di TMR0
		//WriteTimer0(5); // per avere DT=(255-5)*0.2= 50us
		WriteTimer0(0);		// azzero TMR0
		iOVF++;		// conteggio overflow; ogni ovf vale 50us
		// fa lampeggiare LED1 ogni 45 ms; ma IOVF  modificato anche da AscoltaRicev()
		if (iOVF==900)	LED1=!LED1;		
		//iOVFd++;
		LED2=RECEIV;		// replico l'out del receiver su LED2
		StatoReceiv=RECEIV;		//sonda il pin del receiver		
		// if(idurC>=MAXNR) SMstato=SM_STOP;	// buffer overflow
		switch(SMstato){
			case SM_IDLE:	// cerco un gap
				if(StatoReceiv==MARK) {	// finito lo space
					// se l'idle  durato abbastanza  un gap
					if(iOVF<IMPTOUT_TK)		// non  un gap
						iOVF=0;
					else { 
						// il gap  appena finito, ne registro la durata e inizializzo la registrazione
						gapTK=iOVF;
						iOVF=0;
						for(i=0;i<MAXNR;i++) durate[i]=0;	
						idurate=0;		// indice nell'array durate
						idurC=0;		// indice durate registrate (Start + mark dei Commands)
						SMstato=SM_MARK;
					}
				}
				break;
			case SM_MARK:
				if(StatoReceiv==SPACE) {	// finito il mark
					// se lo stato attivo  il mark
					// registro la durata del mark nell'array durate[]
					if(STATOAT==MARK){
						// se  una durata dello Start o di un mark che si riferisce al Command,
						// la scrivo nell'array
						if( (idurate==0) || ((idurate>=SCB) && (idurate<(SCB+NCB)) ) ) 
							if (idurC<MAXNR) durate[idurC++]=iOVF;
					
						idurate++;		// incremento solo per lo StatoAt, quindi ad ogni bit (pi lo Start)
					}	// if(STATOAT==MARK)							
					iOVF=0;
					SMstato=SM_SPACE;
				}
				break;
			case SM_SPACE:
				if(StatoReceiv==MARK) {	// finito lo space
					// se lo stato attivo  lo space
					// ne registro la durata 
					if(STATOAT==SPACE){
						// se  una durata dello Start o che si riferisce al Command,
						// la scrivo nell'array
						if( (idurate==0) || ((idurate>=SCB) && (idurate<(SCB+NCB))) )
							if (idurC<MAXNR) durate[idurC++]=iOVF;
					
						idurate++;		// incremento solo per lo StatoAt, quindi ad ogni bit (pi lo Start)
					}	// if(STATOAT==SPACE)					
					iOVF=0;
					SMstato=SM_MARK;
				}
				else {	//sono ancora in space
					// se  un gap passo in stato di STOP
					if(iOVF>IMPTOUT_TK) SMstato=SM_STOP;
				}
				break;
			case SM_STOP:
				if(StatoReceiv==MARK) {	// finito lo space
					iOVF=0;
				}
				break;			
		}	// switch(SMstato)
	}	// if (INTCONbits.TMR0IF == 1)
}	//hInt_Event
// -- fine ISR --	

void Resume(void){
	SMstato=SM_IDLE;
	idurate=0;
	idurC=0;
//	iOVF=0;
//	WriteTimer0(0);		// azzero TMR0	
}

unsigned char Decode(void){
	// ritorna true solo se si  in stato di SM_STOP e se sono stati ricevuti tutti i bit (idurC==NCB)
	// if ((SMstato==SM_STOP) && (idurC>=(NCB-1)))	return 1;
	if (SMstato==SM_STOP) 	return 1;
	else return 0;
}

void setup(void){
	//------------------- Impostazione porte
	ADCON1=0x0F;			// porte come digital IO
	
	//PORTC bit 6 e 7 per l'USART:RC6=TX=0, RC7=RX=1
	// RC0 input dal receiver; R1,RC2 led  
	LATC=0x00;
	TRISC = 0b10111001;
	
	// RD2..RD7 output per l'LCD
	LATD=0;
	TRISD = 0; 

	// Imposta TMR0
	// TMR0 a 8 bit, T1_PS=1, ovf ogni 255*0.2us =51us;
	OpenTimer0(TIMER_INT_ON & T0_SOURCE_INT & T0_PS_1_1 & T0_8BIT );

		// Configura l'USART
	// 8 bit, 19200 bit/s, 1 bit stop, 0 bit parit
	OpenUSART(USART_TX_INT_OFF & USART_RX_INT_OFF & USART_ASYNCH_MODE & USART_EIGHT_BIT & USART_CONT_RX & USART_BRGH_HIGH, 64);
}

void main (){
	setup();		// impostazioni
	
  	printf("\r\n telec.");
  	printf(SIGLATELEC);
  	printf("\n");

	OpenLCD(20);	// inizializzazione LCD
 	WriteStringLCD ("telec.");
	WriteStringLCD (SIGLATELEC);
	
	// abilito le interruzioni
	RCONbits.IPEN=0;	// modalit compatibile PIC16
 	INTCONbits.GIE=1;	// abilito l'int globale
 	INTCONbits.PEIE=1;	// abilito l'int periferiche

	SMstato=SM_IDLE;
	while (1) {
		
		if (Decode()){
		// siccome la scrittura dura un po'fermo TMR0;
			T0CONbits.TMR0ON=0;		// disabilito TMR0
			// scrivi la sequenza (solo StatoAt del bit iniziale e dei bit del Command)
			ScriviSeq(durate,idurC);
			// decodifica la sequenza; la funzione restituisce l'ascii del tasto
			TastoRil=DecodSeq(durate,Commands);
			printf("\r\n tasto: %c \r\n",TastoRil);
			// scrive sull'LCD
			Line2LCD();
			WriteStringLCD("tasto: ");	
			WriteCharLCD(TastoRil);
			
			T0CONbits.TMR0ON=1;		// riabilito TMR0
			Resume();		// reimposta lo stato a SM_IDLE, cos pu ripartire la registrazione
		}
	}	// while(1)
	
}		//main


void delays_ms(unsigned long cntr) {
	// Delay1KTCYx(1);	  ritardo di 0,2 ms con clock 20 MHz
	for (;cntr>0;cntr--)
		Delay1KTCYx(5);
}
void ScriviSeq(unsigned char durate[],unsigned char ndurate){
	unsigned char i;
	printf(" \r\n ");	// a capo
	for(i=0;i<ndurate;i++){
		printf("%u  ",durate[i]);
	}
}
unsigned char DecodSeq(unsigned char durate[],unsigned char Commands[]){
	// durate[0] contiene lo Start; durate[1] la durata del LSB, durate[NCB] MSB
	// per ogni durate[i] confronta il valore con ONE_TK,ZERO_TK,TOL_TK per vedere se rappresenta 0 o 1
	// se no  n 0 n 1 ritorna 255
	// converte quindi durate[] binario->dec   questo  il Command
	// cerca sequenzialmente Command nell'array Commands[]
	// se lo trova restituisce l'ascii del tasto, altrimenti ritorna 255
	unsigned char Tasto=255,Command=0,i,pot2=1,ibit;
	// ciclo nell'array durate[], da 1 a NCB
	for(i=1;i<=NCB;i++){
		if( Confronta(durate[i],ZERO_TK,TOL_TK) )
			ibit=0;
		else
			if( Confronta(durate[i],ONE_TK,TOL_TK)) ibit=1; else return Tasto;
		Command+=ibit*pot2;
		pot2*=2;
	}	
	// cerca Command in Commands[]
	for(i=0;i<NTASTI;i++)
		if(Commands[i]==Command) break;
	// i primi 10 tasti sono i numeri da 0 a 9; per i successivi il carattere va cercato nell'array TastNN[];
	if(i<10) Tasto=i+48;		// codice ascii del numero
	else if(i<NTASTI) Tasto=TastiNN[i-10];
	return Tasto;
}


unsigned char Confronta(unsigned char cosa,unsigned char concosa,unsigned char Tol){
	if(  (cosa>=(concosa-Tol)) && (cosa<=(concosa+Tol)) ) return 1;
	else return 0;
}





