/* 18_Telecomando
mainRX_SM.c  - PIC del ricevitore
versione implementata come state machine
modulo ricevitore Vishay TSOP34838 (38 kHz)
sezione RX:
out del ricevitore su RC0
led su RC1 lampeggia a frequenza overflow di TMR0
led su RC2 replica l'out del ricevitore
display su PORTD
*/

#pragma config FOSC=HS,LVP=OFF,WDT=OFF
#pragma config PWRT = ON,PBADEN=OFF
#pragma config CCP2MX = ON		//modulo CCP2 su RC1


// -- INCLUDE --
#include <p18f4550.h>
#include <stdio.h>
#include <timers.h>
// usa Delay1KTCYx(n) ritardo (0,2 x n) ms, Delay10KTCYx(n) ritardo (2 x n) ms
#include <delays.h>	
#include <usart.h>

// libreria LCD
void delays_ms(unsigned long);	//richiamata in LCD_44780.c e keyb.c
#define LCD_DEFAULT
#include "LCD_44780.c"
//
// libreria del telecomando
// in telec.h ci sono le specifiche del telecomando e le costanti
// l'unica funz. pubblica  AscoltaRicev() ; qui si usa la costante SIGLATELEC
#define RECEIV PORTCbits.RC0	// ingresso dal modulo receiver (attivo basso)	
// #include "telec.c"

//   da spostare in telec.h		
// -- DEFINE --
#define SONY12

#ifdef SONY12
	#define SIGLATELEC "Sony12"
	#define NTASTI 16
	#define STATOAT 0		// stato attivo mark (livello basso)
	#define ONE_TK 24		//1200us
	#define ZERO_TK 12		//600 us
	#define IMPTOUT_TK 200	// timeout= durata massima impulso= durata minima gap (10ms)
	#define TOL_TK 4		// margine di tolleranza in + e -
	#define SCB 1			// StartCommandBit:dove comincia il command nella squenza impulsi (lo Start  0) 
	#define NCB 7			//n. di bit del command
	//array dei Commands, per i tasti da 0 a 9,#,*,A,B,C,D:
	unsigned char Commands[]={9,0,1,2,3,4,5,6,7,8,35,42,65,66,67,68};
	unsigned char TastiNN[]={'#','*','A','B','C','D'};		
#endif
// DEFINE generici
#define NOCOMMAND 255
#define MAXN 70	// n.max durate impulsi da registrare=dimensione array durate[]		
#define DISP_TK 20000 	// ogni quanti 50us visualizzare i dati (1s)
#define MARK 0
#define SPACE 1 
//   da spostare in telec.h	

#define LED1 LATCbits.LATC1		// 
#define LED2 LATCbits.LATC2		// replica RECEIV
// stati per la state machine
#define SM_IDLE 2
#define SM_MARK 3
#define SM_SPACE 4
#define SM_STOP 5

// -- VARIABILI GLOBALI --
volatile unsigned int iOVF=0,gapTK;	// iOVF contatore di overflow di TMR1
volatile unsigned char i,StatoReceiv,TastoRil,SMstato;
unsigned char idurate=0,idurC=0;
unsigned char durate[MAXN]={0};

// --PROTOTIPI FUNZIONI
void Resume(void);
unsigned char Decode(void);
void ScriviSeq(unsigned char durate[],unsigned char idurate);


// -- ISR - routines interruzioni --
// prototipo funzione che gestisce l'interrupt ad alta priorit
void hInt_Event(void);

// l'interruzione fa saltare alla locazione 0x08, detta interrupt vector 
#pragma code h_int_vect=0x08
void h_int(void) {
	_asm GOTO hInt_Event _endasm
}
#pragma code

// dichiaro che la funzione di gestione interrupt si chiama hInt_Event
#pragma interrupt hInt_Event

// definizione della funzione di gestione int (ad alta priorit, modo compatibile)
void hInt_Event(void) {
	if (INTCONbits.TMR0IF == 1) {	//  avvenuto un overflow in TMR0
		INTCONbits.TMR0IF = 0;	// resetto l'IF di TMR0
		//WriteTimer0(5); // per avere DT=(255-5)*0.2= 50us
		WriteTimer0(0);		// azzero TMR0
		iOVF++;		// conteggio overflow; ogni ovf vale 50us
		// fa lampeggiare LED1 ogni 45 ms; ma IOVF  modificato anche da AscoltaRicev()
		if (iOVF==900)	LED1=!LED1;		
		//iOVFd++;
		LED2=RECEIV;		// replico l'out del receiver su LED2
		StatoReceiv=RECEIV;		//sonda il pin del receiver		
		if(idurate>=MAXN) SMstato=SM_STOP;	// buffer overflow
		switch(SMstato){
			case SM_IDLE:	// cerco un gap
				if(StatoReceiv==MARK) {	// finito lo space
					// se l'idle  durato abbastanza  un gap
					if(iOVF<IMPTOUT_TK)		// non  un gap
						iOVF=0;
					else { 
						// il gap  appena finito, ne registro la durata e inizializzo la registrazione
						gapTK=iOVF;
						iOVF=0;
						for(i=0;i<MAXN;i++) durate[i]=0;	
						idurate=0;		// indice nell'array durate
						//idurC=0;		// indice durate registrate (Start + mark dei Commands)
						SMstato=SM_MARK;
					}
				}
				break;
			case SM_MARK:
				if(StatoReceiv==SPACE) {	// finito il mark
					// registro la durata del mark nell'array durate[]
					durate[idurate++]=iOVF-1;		
					iOVF=0;
					SMstato=SM_SPACE;
				}
				break;
			case SM_SPACE:
				if(StatoReceiv==MARK) {	// finito lo space
					// registro la durata dello space
					durate[idurate++]=iOVF-1;
					iOVF=0;
					SMstato=SM_MARK;
				}
				else {	//sono ancora in space
					// se  un gap passo in stato di STOP
					if(iOVF>IMPTOUT_TK) SMstato=SM_STOP;
				}
				break;
			case SM_STOP:
				if(StatoReceiv==MARK) {	// finito lo space
					iOVF=0;
				}
				break;			
		}	// switch(SMstato)
	}	// if (INTCONbits.TMR0IF == 1)
}	//hInt_Event
// -- fine ISR --	

void Resume(void){
	SMstato=SM_IDLE;
	idurate=0;
}
unsigned char Decode(void){
	if (SMstato==SM_STOP)	return 1;
	else return 0;
}



void setup(void){
	//------------------- Impostazione porte
	ADCON1=0x0F;			// porte come digital IO
	
	//PORTC bit 6 e 7 per l'USART:RC6=TX=0, RC7=RX=1
	// RC0 input dal receiver; R1,RC2 led  
	LATC=0x00;
	TRISC = 0b10111001;
	
	// RD2..RD7 output per l'LCD
	LATD=0;
	TRISD = 0; 

	// Imposta TMR0
	// TMR0 a 8 bit, T1_PS=1, ovf ogni 255*0.2us =51us;
	OpenTimer0(TIMER_INT_ON & T0_SOURCE_INT & T0_PS_1_1 & T0_8BIT );
	
	// abilito le interruzioni
	RCONbits.IPEN=0;	// modalit compatibile PIC16
 	INTCONbits.GIE=1;	// abilito l'int globale
 	INTCONbits.PEIE=1;	// abilito l'int periferiche

		// Configura l'USART
	// 8 bit, 19200 bit/s, 1 bit stop, 0 bit parit
	OpenUSART(USART_TX_INT_OFF & USART_RX_INT_OFF & USART_ASYNCH_MODE & USART_EIGHT_BIT & USART_CONT_RX & USART_BRGH_HIGH, 64);
}

void main (){
	setup();		// impostazioni
	
  	printf("\r\n telec.");
  	printf(SIGLATELEC);
  	printf("\n");

	OpenLCD(20);	// inizializzazione LCD
 	WriteStringLCD ("telec.");
	WriteStringLCD (SIGLATELEC);
	
	SMstato=SM_IDLE;
	while (1) {
		
		/*
		// siccome dura un po'fermo TMR0;
			T0CONbits.TMR0ON=0;		// disabilito TMR0
		*/
		if (Decode()){
			ScriviSeq(durate,idurate);
			Resume();
		}
		/*
			// decodifica la sequenza; la funzione restituisce l'ascii del tasto
			TastoRil=DecodSeq(durate,Commands);
			printf("\r\n tasto: %c \r\n",TastoRil);
			// scrive sull'LCD
			Line2LCD();
			WriteStringLCD("tasto: ");	
			WriteCharLCD(TastoRil);
			
			T0CONbits.TMR0ON=1;		// riabilito TMR0
		*/
		

	}	// while(1)
	
}		//main



void ScriviSeq(unsigned char durate[],unsigned char ndurate){
	unsigned char i;
	printf(" \r\n ");	// a capo
	for(i=0;i<ndurate;i++){
		printf("%u  ",durate[i]);
	}
}


void delays_ms(unsigned long cntr) {
	// Delay1KTCYx(1);	  ritardo di 0,2 ms con clock 20 MHz
	for (;cntr>0;cntr--)
		Delay1KTCYx(5);
}